/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/


#include <QColorDialog>
#include "IHM.h"
#include "props.h"

//#include "EpipolarPickHandler.h"

using namespace PoLAR;

Interface::Interface(): QMainWindow()
{

    // Create a Viewer with shadows management
    viewer = new PoLAR::Viewer(this, "viewer", 0, true);



    //ajout de l'objet phantom
    osg::ref_ptr<osg::Geode> sol = new osg::Geode;
    osg::ref_ptr<osg::ShapeDrawable> shape;
    shape = new osg::ShapeDrawable(new osg::Box(osg::Vec3(3.5f, 2.0f, 0.0f), 7.2f, 4.0f, 0.00001f));
    sol->addDrawable(shape.get());
    osg::ref_ptr<PoLAR::Object3D> object3D = new PoLAR::Object3D(sol.get());
    object3D->setName("sol");
    object3D->setPhantomOn();
    viewer->addObject3D(object3D.get());
    object3D->setEditOn();

    //ajout de la source de lumière
    x=2;
    y=2;
    z=5;
    viewer->addLightSource(x,y,z, true);

    viewer->show();

    viewer->addEventHandler(new osgViewer::StatsHandler);// add stats handler

    _pick = new PoLAR::TabBoxPickHandler; // create pick handler
    _pick->setCharacterSize(15);
    viewer->addEventHandler(_pick);// add pick handler

    PoLAR::DefaultPickHandler *geomPick = new PoLAR::DefaultPickHandler;

    geomPick->setCharacterSize(15);
    viewer->addEventHandler(geomPick);// add geometry pick handler

    toolbar = new QToolBar(this);
    addToolBar(Qt::LeftToolBarArea, toolbar);
    toolbar->setOrientation(Qt::Vertical);

    scrollbarTB = new QToolBar(this);
    scrollbarTB->setOrientation(Qt::Vertical);

    scrollbar = new QScrollBar(this);
    scrollbar->setMinimum(0);
    scrollbar->setMaximum(0);
    scrollbar->setPageStep(10);
    scrollbar->setValue(0);
    scrollbar->setFixedHeight(height());
    scrollbar->setOrientation(Qt::Vertical);
    scrollbarTB->addWidget(scrollbar);


    QObject::connect(scrollbar, SIGNAL(valueChanged(int)), viewer, SLOT(gotoImageSlot(int)));
    QObject::connect(scrollbar, SIGNAL(valueChanged(int)), this, SLOT(changeTitle(int)));

    scrollbarTB->update();
    addToolBar(Qt::RightToolBarArea, scrollbarTB);

    move(0,0);

    QObject::connect(this, SIGNAL(startEditMarkersSignal()), viewer, SLOT(startEditMarkersSlot()));


    /******************************************************************/
    /*           Création de l'interface d'initialisation             */
    /******************************************************************/

    QWidget *widgetInit = new QWidget();
    layoutInit = new QGridLayout(widgetInit);
    toolbar->addWidget(widgetInit);


    //Selection de l'arbre
    QLabel *nArbre = new QLabel("Objet selectionné : ");
    layoutInit->addWidget(nArbre, 1, 0);

    arbreCombo = new QComboBox(this);
    arbreCombo->addItem("arbre");
    arbreCombo->addItem("sapin");

    layoutInit->addWidget(arbreCombo, 1, 1);
    index=1;
    connect(arbreCombo, SIGNAL(currentIndexChanged(int)),this, SLOT(setIndex(int)));



    //Création du slider poour faire pivoter l'objet selon l'angle 1

    QLabel *labAngle1 = new QLabel("Rotation autour de l'axe vert");
    layoutInit->addWidget(labAngle1, 2, 0);
    QSlider *sliderAngle1 = new QSlider(Qt::Horizontal);
    sliderAngle1->setRange(1, 11);
    sliderAngle1->setValue(6);
    layoutInit->addWidget(sliderAngle1, 2, 1, 1, 20);
    angle1prec = 6;

    //Quand la valeur du slider angle1 change, on appel la fonction rotationAngle1
    connect(sliderAngle1, SIGNAL(valueChanged(int)),this, SLOT(rotationAngle1(int)));


    //Création du slider poour faire pivoter une côte selon l'angle 2

    QLabel *labAngle2 = new QLabel("Rotation autour de l'axe bleu");
    layoutInit->addWidget(labAngle2, 3, 0);

    QSlider *sliderAngle2 = new QSlider(Qt::Horizontal);
    sliderAngle2->setRange(1, 11);
    sliderAngle2->setValue(6);
    layoutInit->addWidget(sliderAngle2, 3, 1, 1, 20);
    angle2prec = 6;

    connect(sliderAngle2, SIGNAL(valueChanged(int)),this, SLOT(rotationAngle2(int)));


    //Création du slider poour faire pivoter une côte selon l'angle 3

    QLabel *labAngle3 = new QLabel("Rotation autour de l'axe rouge");
    layoutInit->addWidget(labAngle3, 4, 0);

    QSlider *sliderAngle3 = new QSlider(Qt::Horizontal);
    sliderAngle3->setRange(1, 11);
    sliderAngle3->setValue(6);
    layoutInit->addWidget(sliderAngle3, 4, 1, 1, 20);
    angle3prec = 6;

    connect(sliderAngle2, SIGNAL(valueChanged(int)),this, SLOT(rotationAngle3(int)));


    //Création de la checkbox visible qui en fonction de son etat va afficher l'objet selectionné ou non
    QCheckBox *checkboxVisible = new QCheckBox("Visible", this);
    checkboxVisible->setChecked(true);
    layoutInit->addWidget(checkboxVisible, 5, 0);

    connect(checkboxVisible, SIGNAL(clicked(bool)),this, SLOT(visible(bool)));

    /*Label vide pour créer un saut de ligne, sans ça même si on place le label
     * Lumiere qui suit en position 7,0 , ce dernier se colle à la checkbox*/

    QLabel *saut = new QLabel("    ");
    layoutInit->addWidget(saut, 6, 0);



    QLabel *lumiere = new QLabel("Gestion de la source de lumière");
    layoutInit->addWidget(lumiere, 7, 0);

    //Création de la checkbox On/Off qui va allumer ou éteindre la source de lumière
    QCheckBox *checkboxLum = new QCheckBox("On/Off", this);
    checkboxLum->setChecked(true);
    layoutInit->addWidget(checkboxLum, 8, 0);
     connect(checkboxLum, SIGNAL(clicked(bool)),this, SLOT(lumEtat(bool)));

     //Création de la checkbox Visible/Cachée
     QCheckBox *checkboxLumVisible = new QCheckBox("Visible/Cachée", this);
     checkboxLumVisible->setChecked(false);
     layoutInit->addWidget(checkboxLumVisible, 8, 1);
     connect(checkboxLumVisible, SIGNAL(clicked(bool)),this, SLOT(lumVisible(bool)));

     QLabel *saut1 = new QLabel("");
     layoutInit->addWidget(saut1, 9, 0);

     QLabel *position = new QLabel("Voici les racourcis pour ");
     layoutInit->addWidget(position, 10, 0);

     QLabel *position1 = new QLabel("le changement de position : ");
     layoutInit->addWidget(position1, 11, 0);

     QLabel *positionHaut = new QLabel("Vers le haut : Z  ");
     layoutInit->addWidget(positionHaut, 12, 0);

     QLabel *positionBas = new QLabel("Vers le bas : S");
     layoutInit->addWidget(positionBas, 12, 1);

     QLabel *positionGauche = new QLabel("Vers la gauche : Q  ");
     layoutInit->addWidget(positionGauche, 13, 0);

     QLabel *positionDroite = new QLabel("Vers la droite : Z");
     layoutInit->addWidget(positionDroite, 13, 1);

     QLabel *positionAvant = new QLabel("Vers l'avant : A");
     layoutInit->addWidget(positionAvant, 14, 0);

     QLabel *positionArriere = new QLabel("Vers l'arrière : E");
     layoutInit->addWidget(positionArriere, 14, 1);

     QLabel *saut2 = new QLabel("");
     layoutInit->addWidget(saut2, 15, 0);

     QLabel *autre = new QLabel("Autres racourcis clavier : ");
     layoutInit->addWidget(autre, 16, 0);

     QLabel *w = new QLabel("Activer le picking : W");
     layoutInit->addWidget(w, 17, 0);

     QLabel *b = new QLabel("Afficher/Cacher l'image de fond : B");
     layoutInit->addWidget(b, 17, 1);

    // add focus to this interface in order to get the keyboard events
    this->setFocusPolicy(Qt::StrongFocus);
    // remove the focus from the Viewer, otherwise it will bypass the interface's events
    viewer->setFocusPolicy(Qt::NoFocus);
}

Interface::~Interface()
{
    delete lightDialog;
    delete mb;
}

void Interface::changeTitle(int n)
{
    char buf[256];
    sprintf(buf, "Image %3d",n);
    statuslabel->setText(*(new QString(buf)));
}

//Allume ou non la source de limière
void Interface::lumEtat(bool state)
{

    if(state==true)
    {

        viewer->setShadowsOn();
        //viewer->getLightSource(1)->setOn();

    }

    else
    {
       viewer->setShadowsOff();
    }
}

//Affiche ou non la source de lumi_re
void Interface::lumVisible(bool state)
{

    if(state==true)
    {


        viewer->getLightSource(1)->setViewable(true);

    }

    else
    {
       viewer->getLightSource(1)->setViewable(false);
    }
}

void Interface::posLumiereHaut()
{
    z+=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

void Interface::posLumiereBas()
{
    z-=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

void Interface::posLumiereDroite()
{
    x+=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

void Interface::posLumiereGauche()
{
    x-=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

void Interface::posLumiereAvant()
{
    y+=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

void Interface::posLumiereArriere()
{
    y-=1;
    viewer->getLightSource(1)->setPosition(x,y,z,1);

}

//Affiche ou non l'objet en fonction de la checkboxVisible
void Interface::visible(bool state)
{
    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);
    if(state==true && obj->isDisplayed()==false)
    {
        obj->setDisplayOn();
    }

    else
    {
        if(state==false && obj->isDisplayed()==true)
        {
            obj->setDisplayOff();
        }
    }
}


//Met l'index à l'entier correspondant à l'objet seclectionné dans le menu deroulant et remet les sliders à leurs valeurs min
void Interface::setIndex(int i)
{
    index=i+1;
}


//Fait pivoter une cote en fonction de la modification de la position du SLider
void Interface::rotationAngle1(int i)
{

    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);

    if(angle1prec<i)
    {

        obj->rotate(M_PI/10,0,1,0);

        angle1prec=i;
    }

    else
    {

        obj->rotate(-M_PI/10,0,1,0);

        angle1prec=i;
    }
}


//Fait pivoter une cote en fonction de la modification de la position du SLider
void Interface::rotationAngle2(int i)
{

    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);

    if(angle2prec<i)
    {
        obj->rotate(M_PI/10,0,0,1);
        angle2prec=i;
    }

    else
    {
        obj->rotate(-M_PI/10,0,0,1);
        angle2prec=i;
    }
}

//Fait pivoter une cote en fonction de la modification de la position du SLider
void Interface::rotationAngle3(int i)
{

    osg::ref_ptr<PoLAR::Object3D> obj =   viewer->getObject3D(index);

    if(angle3prec<i)
    {
        obj->rotate(M_PI/10,1,0,0);
        angle3prec=i;
    }

    else
    {
        obj->rotate(-M_PI/10,1,0,0);
        angle3prec=i;
    }
}

// Add key bindings
void Interface::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
    //case Qt::Key_Q: this->exit(); break;
    case Qt::Key_B:
    {
        // Hide/display background image
        viewer->bgImageToggle();
        break;
    }
    case Qt::Key_W:
    {
        // Set the scene graph interaction to be active
        viewer->startManipulateSceneSlot();
        break;
    }

    case Qt::Key_Q:
    {
        // Déplace la source de lumière vers la gauche
        posLumiereGauche();
        break;
    }

    case Qt::Key_D:
    {
        // Déplace la source de lumière vers la droite
        posLumiereDroite();
        break;
    }
    case Qt::Key_Z:
    {
        // Déplace la source de lumière vers le haut
        posLumiereHaut();
        break;
    }

    case Qt::Key_S:
    {
        // Déplace la source de lumière vers le bas
        posLumiereBas();
        break;
    }
    case Qt::Key_A:
    {
        // Déplace la source de lumière vers le bas
        posLumiereAvant();
        break;
    }
    case Qt::Key_E:
    {
        // Déplace la source de lumière vers le bas
        posLumiereArriere();
        break;
    }

    default:
        viewer->notify(event);
    }
}

void Interface::keyReleaseEvent(QKeyEvent *event)
{
    //simply forwarding to the viewer
    viewer->notify(event);
}
