﻿/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/

#include <QApplication>
#include <PoLAR/AnimatedObject3D.h>
#include "IHM.h"
#include "props.h"
#include "MyViewer.h"
#include <PoLAR/Image.h>
#include <PoLAR/Util.h>
#include <osg/Geode>
#include <osg/MatrixTransform>
#include <osg/Geometry>
#include <osg/ShapeDrawable>
#include <osg/StateSet>
#include <osg/ColorMask>
#include <osg/PolygonMode>
#include <osgDB/ReadFile>
#include <osgDB/WriteFile>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osg/Texture2D>
#include <iostream>
#include <PoLAR/FrameAxis.h>
#include <PoLAR/Light.h>
#include <osg/Vec4>


int main(int argc,char ** argv)
{
    QApplication app(argc,argv);
    Interface inter;



    osg::ref_ptr<PoLAR::Image_uc> myImage;




    // Charge l'image de fond

    if(PoLAR::Util::fileExists("../data/fond1.jpg"))
    {
        myImage = new PoLAR::Image_uc("../data/fond1.jpg", true);
    }
    else
    {
        std::cerr << "../data/fond1.jpg" << ": file not found" << std::endl;
        exit(0);
    }

    // Ajoute l'image chargée au fond
    inter.addImage(myImage, 1);
    double width = myImage->getWidth();
    double height = myImage->getHeight();
    inter.getViewer()->resize(width,height);
    PoLAR::Viewer::ProjectionType pt=PoLAR::Viewer2D3D::VISION;





    // Charge la matrice de projection
    osg::Matrixd P = PoLAR::Util::readProjectionMatrix("../data/matrice.txt");
    inter.setProjection(P, pt);




    // Ajout des axes ( rouge-> X, vert-> y et bleu-> z)
    PoLAR::FrameAxis *polarAxis= new PoLAR::FrameAxis;
    polarAxis->setLightingOn();
    PoLAR::Object3D *axis = new PoLAR::Object3D(polarAxis,true);
    axis->setName("FrameAxis");
    axis->setEditOn();
    axis->translate(0,0,0);
    axis->scale(1.0f);
    inter.addNode(axis);





    // Ajout de l'arbre1 non animé
    if(!PoLAR::Util::fileExists("../data/arbre1.obj"))
    {
        std::cerr << "Unable to load model" << std::endl;
        exit(0);
    }
    else
    {

        osg::ref_ptr<PoLAR::Object3D> arbre1 = new PoLAR::Object3D("../data/arbre1.obj", true, true, true);
        osgUtil::SmoothingVisitor smooth;
        osgUtil::Optimizer optimizer;
        optimizer.optimize(arbre1.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
        arbre1->accept(smooth);
        arbre1->setName("arbre1");
        arbre1->translate(1.8,2.5,-5.8);
        arbre1->scale(0.2f);
        inter.addNode(arbre1);
        inter.getViewer()->setTrackNode(arbre1.get());

    }



    // Ajout de l'arbre2 non animé
    if(!PoLAR::Util::fileExists("../data/arbre2.obj"))
    {
        std::cerr << "Unable to load model" << std::endl;
        exit(0);
    }
    else
    {

        osg::ref_ptr<PoLAR::Object3D> arbre2 = new PoLAR::Object3D("../data/arbre2.obj", true, true, true);
        osgUtil::SmoothingVisitor smooth;
        osgUtil::Optimizer optimizer;
        optimizer.optimize(arbre2.get(), osgUtil::Optimizer::ALL_OPTIMIZATIONS);
        arbre2->accept(smooth);
        arbre2->setName("arbre2");
        arbre2->translate(4.5,0,-5.5);
        arbre2->scale(0.2f);
        inter.addNode(arbre2);
        inter.getViewer()->setTrackNode(arbre2.get()); // set this object to be tracked by the manipulator

    }




    //ajout de l'arbre 1 animé

    // if(!PoLAR::Util::fileExists("../data/arbre1.osgt"))
    // {
    //     std::cerr << "Unable to load model" << std::endl;
    //     exit(0);
    // }
    // else
    // {

    //     PoLAR::Viewer viewer;
    //     osg::ref_ptr<PoLAR::AnimatedObject3D> arbre1 = new PoLAR::AnimatedObject3D("../data/arbre1.osgt");
    //     arbre1->translate(0,0,-7);
    //     arbre1->scale(0.2f);
    //     viewer.addObject3D(arbre1.get());


    // }
    

    inter.getViewer()->center();
    inter.show();



    return app.exec();
}
